asm pillbox_2

import StandardLibrary
import CTLlibrary
import LTLlibrary

// Third refinement level of the Pill Box: We consider three compartment, and time controlled By the pillbox 

signature:
	//*************************************************
	// DOMAINS
	//*************************************************
	abstract domain Compartment
	
	enum domain LedLights = {OFF | ON | BLINKING}
	
	enum domain OutMessages = {
		TAKE_ASPIRINE | TAKE_TYLENOL | TAKE_MOMENT | 
		NONE | 
		ASPIRINE_MISSED | TYLENOL_MISSED | MOMENT_MISSED | 
		ASPIRINE_COMPARTMENT_NOT_CLOSED | TYLENOL_COMPARTMENT_NOT_CLOSED | MOMENT_COMPARTMENT_NOT_CLOSED |
		CLOSE_ASPIRINE_COMPARTMENT_IN_10_MIN | CLOSE_TYLENOL_COMPARTMENT_IN_10_MIN | CLOSE_MOMENT_COMPARTMENT_IN_10_MIN |
		TAKE_ASPIRINE_IN_10_MIN | TAKE_TYLENOL_IN_10_MIN | TAKE_MOMENT_IN_10_MIN 
	}
	
	enum domain Drugs = {
		ASPIRINE | TYLENOL | MOMENT
	}
	
	domain Time subsetof Integer
	
	//*************************************************
	// FUNCTIONS
	//*************************************************
	dynamic monitored openSwitch: Compartment -> Boolean
	dynamic controlled opened: Compartment -> Boolean
	dynamic controlled redLed: Compartment -> LedLights
	dynamic controlled outMess: Compartment -> OutMessages
	dynamic controlled logMess: Compartment -> OutMessages
	dynamic controlled time_consumption: Compartment -> Time
	dynamic controlled requestSatisfied: Compartment -> Boolean	
	dynamic controlled name: Compartment -> Drugs

	// The systemTime is expressed as the number of hours passed since the 01/01/1970
	dynamic controlled systemTime: Time
	dynamic controlled compartmentTimer: Compartment -> Time
	
	static tenMinutes: Time
	
	//*************************************************
	// STATIC VARIABLES
	//*************************************************
	static compartment1: Compartment
	static compartment2: Compartment
	static compartment3: Compartment	
	
definitions:
	//*************************************************
	// DOMAIN DEFINITIONS
	//*************************************************
	domain Time = {0..10}
	
	//*************************************************
	// FUNCTIONS DEFINITIONS
	//*************************************************
	function tenMinutes = 1
	
	//*************************************************
	// RULE DEFINITIONS
	//*************************************************
	// Rule that implement the writing on the log file
	rule r_writeToFile($compartment in Compartment) = skip
	
	// Rule to set the led red ON when the pill has to be taken
	rule r_pillToBeTaken($compartment in Compartment) =
		par
			if redLed($compartment) != ON then compartmentTimer($compartment) := systemTime endif
			redLed($compartment) := ON 
			if ($compartment=compartment1) then
				outMess($compartment) := TAKE_TYLENOL
			else 
				if ($compartment=compartment2) then
					outMess($compartment) := TAKE_ASPIRINE
				else
					outMess($compartment) := TAKE_MOMENT
				endif
			endif			
		endpar	
		
	// Rule to set the red led blinking, after the compartment opening
	rule r_compartmentOpened($compartment in Compartment) =
		par
			if redLed($compartment) != BLINKING and outMess($compartment) != CLOSE_ASPIRINE_COMPARTMENT_IN_10_MIN and outMess($compartment) != CLOSE_TYLENOL_COMPARTMENT_IN_10_MIN and 
				outMess($compartment) != CLOSE_MOMENT_COMPARTMENT_IN_10_MIN then compartmentTimer($compartment) := systemTime endif
			redLed($compartment) := BLINKING
			if ($compartment=compartment1) then
				outMess($compartment) := CLOSE_TYLENOL_COMPARTMENT_IN_10_MIN
			else 
				if ($compartment=compartment2) then
					outMess($compartment) := CLOSE_ASPIRINE_COMPARTMENT_IN_10_MIN
				else
					outMess($compartment) := CLOSE_MOMENT_COMPARTMENT_IN_10_MIN
				endif
			endif				
		endpar
		
	// Rule to take the system back to the initial state when the compartment is open and redLed is ON
	rule r_pillTaken_compartmentOpened($compartment in Compartment) =
		par
			redLed($compartment) := OFF
			outMess($compartment) := NONE
			compartmentTimer($compartment) := systemTime
			requestSatisfied($compartment) := true
		endpar
		
	// Rule to handle the closing of a compartment when the RED Led is blinking
	rule r_compartmentClosed($compartment in Compartment) =
		par
			redLed($compartment) := OFF
			outMess($compartment) := NONE
			compartmentTimer($compartment) := systemTime
			requestSatisfied($compartment) := true
		endpar
		
	// Rule to take the system back to the initial state when the Blinking timeout is passed
	rule r_timeOutExpired_missedPill($compartment in Compartment) =
		par
			redLed($compartment) := OFF
			outMess($compartment) := NONE
			if ($compartment=compartment1) then
				logMess($compartment) := TYLENOL_MISSED
			else 
				if ($compartment=compartment2) then
					logMess($compartment) := ASPIRINE_MISSED
				else
					logMess($compartment) := MOMENT_MISSED
				endif
			endif	
			compartmentTimer($compartment) := systemTime
			requestSatisfied($compartment) := true
			r_writeToFile[$compartment] 
		endpar	
		
	// Rule to take the system back to the initial state when the Blinking timeout is passed and the compartment has not been closed
	rule r_timeOutExpired_compartmentOpened($compartment in Compartment) =
		par
			redLed($compartment) := OFF
			outMess($compartment) := NONE
			if ($compartment=compartment1) then
				logMess($compartment) := TYLENOL_COMPARTMENT_NOT_CLOSED 
			else 
				if ($compartment=compartment2) then
					logMess($compartment) := ASPIRINE_COMPARTMENT_NOT_CLOSED
				else
					logMess($compartment) := MOMENT_COMPARTMENT_NOT_CLOSED
				endif
			endif	
			compartmentTimer($compartment) := systemTime
			requestSatisfied($compartment) := true
			r_writeToFile[$compartment]
		endpar	
		
	// Rule to set the red led blinking, after the compartment opening
	rule r_takeInTimeout($compartment in Compartment) =
		par
			if redLed($compartment) != BLINKING and outMess($compartment) != TAKE_ASPIRINE_IN_10_MIN and outMess($compartment) != TAKE_TYLENOL_IN_10_MIN and 
				outMess($compartment) != TAKE_MOMENT_IN_10_MIN then compartmentTimer($compartment) := systemTime endif
			redLed($compartment) := BLINKING
			if ($compartment=compartment1) then
				outMess($compartment) := TAKE_TYLENOL_IN_10_MIN	
			else 
				if ($compartment=compartment2) then
					outMess($compartment) := TAKE_ASPIRINE_IN_10_MIN	
				else
					outMess($compartment) := TAKE_MOMENT_IN_10_MIN	
				endif
			endif			
		endpar
	
	//*************************************************
	// Property Verification
	//*************************************************
	// If the pill has to be taken, red led will lights up
	CTLSPEC ag((time_consumption(compartment1)<systemTime and not requestSatisfied(compartment1) and redLed(compartment1) = OFF) implies ax(redLed(compartment1) = ON))
	CTLSPEC ag((time_consumption(compartment2)<systemTime and not requestSatisfied(compartment2) and redLed(compartment2) = OFF) implies ax(redLed(compartment2) = ON))
	CTLSPEC ag((time_consumption(compartment3)<systemTime and not requestSatisfied(compartment3) and redLed(compartment3) = OFF) implies ax(redLed(compartment3) = ON))
	// If the patient does not take the pill or the compartment has to be closed, the red light will blink
	CTLSPEC ag(((redLed(compartment1) = ON and not opened(compartment1) and openSwitch(compartment1)) or (redLed(compartment1) = ON and systemTime-compartmentTimer(compartment1)=tenMinutes and not(opened(compartment1)))) implies ax(redLed(compartment1) = BLINKING))
	CTLSPEC ag(((redLed(compartment2) = ON and not opened(compartment2) and openSwitch(compartment2)) or (redLed(compartment2) = ON and systemTime-compartmentTimer(compartment2)=tenMinutes and not(opened(compartment2)))) implies ax(redLed(compartment2) = BLINKING))
	CTLSPEC ag(((redLed(compartment3) = ON and not opened(compartment3) and openSwitch(compartment3)) or (redLed(compartment3) = ON and systemTime-compartmentTimer(compartment3)=tenMinutes and not(opened(compartment3)))) implies ax(redLed(compartment3) = BLINKING))
	// The red light will change value after 10 minutes if the patient doesn't take the pill
	CTLSPEC ag((redLed(compartment1) = ON and systemTime-compartmentTimer(compartment1)=tenMinutes and not(opened(compartment1))) implies ax(redLed(compartment1) = BLINKING))
	CTLSPEC ag((redLed(compartment2) = ON and systemTime-compartmentTimer(compartment2)=tenMinutes and not(opened(compartment2))) implies ax(redLed(compartment2) = BLINKING))
	CTLSPEC ag((redLed(compartment3) = ON and systemTime-compartmentTimer(compartment3)=tenMinutes and not(opened(compartment3))) implies ax(redLed(compartment3) = BLINKING))
	CTLSPEC ag((redLed(compartment1) = BLINKING and systemTime-compartmentTimer(compartment1)>tenMinutes and not(openSwitch(compartment1))) implies ax(redLed(compartment1) = OFF))
	CTLSPEC ag((redLed(compartment2) = BLINKING and systemTime-compartmentTimer(compartment2)>tenMinutes and not(openSwitch(compartment2))) implies ax(redLed(compartment2) = OFF))
	CTLSPEC ag((redLed(compartment3) = BLINKING and systemTime-compartmentTimer(compartment3)>tenMinutes and not(openSwitch(compartment3))) implies ax(redLed(compartment3) = OFF))
	// If the patient takes the pill, red light will turn-off
	CTLSPEC (forall $d in Compartment with ag((time_consumption($d) <systemTime and not requestSatisfied($d) and opened($d) and not(openSwitch($d)) and not(redLed($d) = OFF) and not(systemTime-compartmentTimer($d)>=tenMinutes)) implies ax(redLed($d) = OFF)))
	//CTLSPEC ag((time_consumption(compartment2)<systemTime and not requestSatisfied(compartment2) and opened(compartment2) and not(openSwitch(compartment2)) and not(redLed(compartment2) = OFF) and not(systemTime-compartmentTimer(compartment2)>=tenMinutes)) implies ax(redLed(compartment2) = OFF))
	//CTLSPEC ag((time_consumption(compartment3)<systemTime and not requestSatisfied(compartment3) and opened(compartment3) and not(openSwitch(compartment3)) and not(redLed(compartment3) = OFF) and not(systemTime-compartmentTimer(compartment3)>=tenMinutes)) implies ax(redLed(compartment3) = OFF))
		
	//*************************************************
	// MAIN Rule
	//*************************************************
	// MAIN Rule
	main rule r_Main =
		par
			// Set the SystemTime
			systemTime := (systemTime + 1) mod 10
			
			forall $compartment in Compartment do
				if (not(requestSatisfied($compartment)) and systemTime < 10) then
					par					
						// Set the satus of the compartment
						if not opened($compartment) and openSwitch($compartment) then opened($compartment) := true endif
						if opened($compartment) and not openSwitch($compartment) then opened($compartment) := false endif
						
						// Starting from the IDLE state, the pill has to be taken 
						if redLed($compartment) = OFF then if (time_consumption($compartment)<=systemTime and not requestSatisfied($compartment)) then r_pillToBeTaken[$compartment] endif endif
						// It is open, drug to be taken, it becomes closed
						if redLed($compartment) = ON and not(systemTime-compartmentTimer($compartment)>=tenMinutes) and opened($compartment) and not openSwitch($compartment) then r_pillTaken_compartmentOpened[$compartment] endif
						// It is closed drug to be taken and it becomes open     
						if redLed($compartment) = ON and not opened($compartment) and openSwitch($compartment) then r_compartmentOpened[$compartment] endif
						// It is closed drug to be taken and timeout     
						if redLed($compartment) = ON then 
							if systemTime-compartmentTimer($compartment)>=tenMinutes then 
								if opened($compartment) then r_compartmentOpened[$compartment] else if not openSwitch($compartment) then r_takeInTimeout[$compartment] endif endif 
							endif 
						endif
						// It is blinking, and it becomes closed (or remains closed) or timeout   
						if redLed($compartment) = BLINKING then 
							if not openSwitch($compartment) and opened($compartment) then r_compartmentClosed[$compartment] 
							else 
								if systemTime-compartmentTimer($compartment)>tenMinutes then
									if openSwitch($compartment) then r_timeOutExpired_compartmentOpened[$compartment] else r_timeOutExpired_missedPill[$compartment] endif
								else 
									if openSwitch($compartment) then r_compartmentOpened[$compartment] endif
								endif 
							endif 
						endif
					endpar
				endif
		endpar
			
default init s0:	
	// Controlled function that indicates the status of the compartment
	function opened($compartment in Compartment) = false		
		
	// Reset the output display message and the log message
	function outMess($compartment in Compartment) = NONE
	function logMess($compartment in Compartment) = NONE
	
	// Turn-off all the led of the Compartments
	function redLed($compartment in Compartment) = OFF
	
	// Each compartment's timer starts from 0
	function compartmentTimer($compartment in Compartment) = 0

	// Initialization of the time consumption
	function time_consumption($compartment in Compartment) =
		switch($compartment)
			case compartment1 : 1
			case compartment2 : 3
			case compartment3 : 2
		endswitch

	// Initialization of the SystemTime
	function systemTime = 0
	
	// At the initialization phase, every drug has to be assumed
	function requestSatisfied($compartment in Compartment) = false
	
	// Insert a drug in each compartment	
	function name($compartment in Compartment) =
		switch($compartment)
			case compartment1 : TYLENOL
			case compartment2 : ASPIRINE
			case compartment3 : MOMENT
		endswitch