asm pillbox_FULL

import StandardLibrary
import CTLlibrary
import LTLlibrary

// Third refinement level of the Pill Box: We consider three compartment, and time controlled By the pillbox 

signature:
	//*************************************************
	// DOMAINS
	//*************************************************
	abstract domain Compartment
	
	enum domain LedLights = {OFF | ON | BLINKING}

	//*************************************************
	// FUNCTIONS
	//*************************************************
	dynamic monitored openSwitch: Compartment -> Boolean
	dynamic controlled opened: Compartment -> Boolean
	dynamic controlled redLed: Compartment -> LedLights
	dynamic controlled outMess: Compartment -> String
	dynamic controlled logMess: Compartment -> String
	dynamic controlled time_consumption: Compartment -> Seq(Natural)
	dynamic controlled name: Compartment -> String
	dynamic controlled drugIndex: Compartment -> Natural

	// The systemTime is expressed as the number of hours passed since the 01/01/1970
	dynamic monitored systemTime: Natural
	dynamic controlled compartmentTimer: Compartment -> Natural
	static tenMinutes: Integer
	
	//*************************************************
	// STATIC VARIABLES
	//*************************************************
	static compartment1: Compartment
	static compartment2: Compartment
	static compartment3: Compartment	
	
definitions:
	//*************************************************
	// FUNCTIONS DEFINITIONS
	//*************************************************
	function tenMinutes = 600

	//*************************************************
	// RULE DEFINITIONS
	//*************************************************
	// Rule that implement the writing on the log file
	rule r_writeToFile($compartment in Compartment) = skip
	
	// Rule to set the led red ON when the pill has to be taken
	rule r_pillToBeTaken($compartment in Compartment) =
		par
			if redLed($compartment) != ON then compartmentTimer($compartment) := systemTime endif
			redLed($compartment) := ON 
			outMess($compartment) := "Take " + name($compartment)			
		endpar	
		
	// Rule to set the red led blinking, after the compartment opening
	rule r_compartmentOpened($compartment in Compartment) =
		par
			if redLed($compartment) != BLINKING and outMess($compartment) != ("Close " + name($compartment) + " in 10 minutes") then compartmentTimer($compartment) := systemTime endif
			redLed($compartment) := BLINKING
			outMess($compartment) := "Close " + name($compartment) + " in 10 minutes"				
		endpar
		
	// Rule to take the system back to the initial state when the compartment is open and redLed is ON
	rule r_pillTaken_compartmentOpened($compartment in Compartment) =
		par
			redLed($compartment) := OFF
			outMess($compartment) := ""
			compartmentTimer($compartment) := systemTime
			drugIndex($compartment) := drugIndex($compartment) + 1n
		endpar
		
	// Rule to handle the closing of a compartment when the RED Led is blinking
	rule r_compartmentClosed($compartment in Compartment) =
		par
			redLed($compartment) := OFF
			outMess($compartment) := ""
			compartmentTimer($compartment) := systemTime
			drugIndex($compartment) := drugIndex($compartment) + 1n
		endpar
		
	// Rule to take the system back to the initial state when the Blinking timeout is passed
	rule r_timeOutExpired_missedPill($compartment in Compartment) =
		par
			redLed($compartment) := OFF
			outMess($compartment) := ""
			logMess($compartment) := name($compartment) + " missed"	
			compartmentTimer($compartment) := systemTime
			drugIndex($compartment) := drugIndex($compartment) + 1n
			r_writeToFile[$compartment] 
		endpar	
		
	// Rule to take the system back to the initial state when the Blinking timeout is passed and the compartment has not been closed
	rule r_timeOutExpired_compartmentOpened($compartment in Compartment) =
		par
			redLed($compartment) := OFF
			outMess($compartment) := ""
			logMess($compartment) := name($compartment) + " compartment not closed"
			compartmentTimer($compartment) := systemTime
			drugIndex($compartment) := drugIndex($compartment) + 1n
			r_writeToFile[$compartment]
		endpar	
		
	// Rule to set the red led blinking, after the compartment opening
	rule r_takeInTimeout($compartment in Compartment) =
		par
			if redLed($compartment) != BLINKING and outMess($compartment) != ("Take " + name($compartment) + " in 10 minutes") then compartmentTimer($compartment) := systemTime endif
			redLed($compartment) := BLINKING
			outMess($compartment) := "Take " + name($compartment) + " in 10 minutes"		
		endpar
		
	//*************************************************
	// MAIN Rule
	//*************************************************	
	// MAIN Rule
	main rule r_Main =
		forall $compartment in Compartment do
			if (drugIndex($compartment)<iton(length(time_consumption($compartment)))) then
				par					
					// Set the satus of the compartment
					if not opened($compartment) and openSwitch($compartment) then opened($compartment) := true endif
					if opened($compartment) and not openSwitch($compartment) then opened($compartment) := false endif
					
					// Starting from the IDLE state, the pill has to be taken 
					if redLed($compartment) = OFF then if (at(time_consumption($compartment),drugIndex($compartment))<systemTime) then r_pillToBeTaken[$compartment] endif endif
					// It is open, drug to be taken, it becomes closed
					if redLed($compartment) = ON and not(systemTime-compartmentTimer($compartment)>=tenMinutes) and opened($compartment) and not openSwitch($compartment) then r_pillTaken_compartmentOpened[$compartment] endif
					// It is closed drug to be taken and it becomes open     
					if redLed($compartment) = ON and not opened($compartment) and openSwitch($compartment) then r_compartmentOpened[$compartment] endif
					// It is closed drug to be taken and timeout     
					if redLed($compartment) = ON then 
						if systemTime-compartmentTimer($compartment)>=tenMinutes then 
							if opened($compartment) then r_compartmentOpened[$compartment] else if not openSwitch($compartment) then r_takeInTimeout[$compartment] endif endif 
						endif 
					endif
					// It is blinking, and it becomes closed (or remains closed) or timeout   
					if redLed($compartment) = BLINKING then 
						if not openSwitch($compartment) and opened($compartment) then r_compartmentClosed[$compartment] 
						else 
							if systemTime-compartmentTimer($compartment)>tenMinutes then
								if openSwitch($compartment) then r_timeOutExpired_compartmentOpened[$compartment] else r_timeOutExpired_missedPill[$compartment] endif
							else 
								if openSwitch($compartment) then r_compartmentOpened[$compartment] endif
							endif 
						endif 
					endif
				endpar
			endif
			
default init s0:	
	// Controlled function that indicates the status of the compartment
	function opened($compartment in Compartment) = false		
		
	// Reset the output display message and the log message
	function outMess($compartment in Compartment) = ""
	function logMess($compartment in Compartment) = ""
	
	// Turn-off all the led of the Compartments
	function redLed($compartment in Compartment) = OFF
	
	// Each compartment's timer starts from 0
	function compartmentTimer($compartment in Compartment) = 0n

	// Initialization of the time consumption
	function time_consumption($compartment in Compartment) =
		switch($compartment)
			case compartment1 : [1n, 20n, 30n]
			case compartment2 : [40n, 50n, 60n]
			case compartment3 : [9n, 20n, 30n]
		endswitch

	// Initialization of the SystemTime
	function systemTime = 0n
	
	// Insert a drug in each compartment	
	function name($compartment in Compartment) =
		switch($compartment)
			case compartment1 : "Tylenol"
			case compartment2 : "Aspirine"
			case compartment3 : "Moment"
		endswitch
		
	// Every compartment has an index starting from 0
	function drugIndex($compartment in Compartment) = 0n